clear all; clc;

% -------------------------------------------------------------------------
% Define constants
% -------------------------------------------------------------------------
c0 = 299792458;                 % speed of light
me = 9.1e-31;                   % electron mass
e = 1.6e-19;                    % electron charge
a = -2e49;                       % assymetry parameter               << 1e39 - 1e52
gam = 1e14;                     % damping parameter                 <<
w0 = (c0 / 200e-9 * 2 * pi);    % resonance frequency               <<
eps0 = 8.85e-12;                % permittivity in vacuum
I_at = 3.5e20;                  % characteristic intensity
E_at = sqrt(2 * I_at / eps0 / c0); % field strength

% -------------------------------------------------------------------------
% Laser parameters
% -------------------------------------------------------------------------
lambda1 = 800e-9;

w1 = c0 / 800e-9 * 2 * pi;
w2 = 0.8*w1;
w3 = 1.2*w1;
% w1list = linspace(0.8 * w0 , 1.2 * w0, 32);

% -------------------------------------------------------------------------
% Time and frequency grids
% -------------------------------------------------------------------------
Nt = 1024 * 64 * 16;
t = linspace(-50e-15, 50e-15, Nt);
dt = t(2) - t(1);
df = 1/Nt/dt;
w = [[0 : Nt/2-1], [-Nt/2 : -1]] * df * 2 * pi;

lambda = c0 ./ w / 2 / pi;
%--------------------------------------------------------------------------
% Plot potential
%--------------------------------------------------------------------------
xx = linspace(-10e-10, 10e-10, 100);
Uxs = 1/2*w0.^2*xx.^2;                     % Symetric potential
Uxa = 1/2*w0.^2*xx.^2 + 1/4*a*xx.^4;       % Asymmetric potential

figure(2);
plot(xx/1e-10, Uxs, '--', xx/1e-10, Uxa, 'k');
title('U(x)'); xlabel('Position (ang.)'); ylabel('U(x)');


    % -------------------------------------------------------------------------
    % Laser field
    % -------------------------------------------------------------------------
    E = E_at / 5 * exp(-2*log(2)*t.^2 / 30e-15.^2) .* cos(w1*t) + ...           % <<
        E_at / 5 * exp(-2*log(2)*t.^2 / 30e-15.^2) .* cos(w2*t) + ...           % <<
        E_at / 5 * exp(-2*log(2)*t.^2 / 30e-15.^2) .* cos(w3*t);                % <<
    
    % -------------------------------------------------------------------------
    % Memory allocation
    % -------------------------------------------------------------------------
    ddx = zeros(1, Nt);             % acceleration
    dx = zeros(1, Nt);              % velocity
    x = zeros(1, Nt);               % position
    
    % -------------------------------------------------------------------------
    % Solve the equation of motion using numerical integration
    % -------------------------------------------------------------------------
    for tidx = 1 : Nt-1
        x(tidx + 1) = x(tidx) + dt * dx(tidx);
        dx(tidx + 1) = dx(tidx) + dt * ddx(tidx);
        ddx(tidx + 1) = - 2 * gam * dx(tidx+1) - w0^2*x(tidx+1) - a*x(tidx+1).^3 - e * E(tidx + 1) / me;  % <<
    end
    
    Cn = fft(fftshift(x));
    [vv idx] = min(abs(w - w1));
%     Cn1s(w1idx) = Cn(idx);
    
    % -------------------------------------------------------------------------
    % Plot results
    % -------------------------------------------------------------------------
    figure(1); clf;
    subplot(2, 2, 1);
    plot(t, E/max(E), 'r', t, x/1e-10, 'b'); grid on;
    title('E(t)');  xlabel('Time (s)'); ylabel('x (ang.)'); ylim([-3 3]);
    legend({'E', 'x'});
    
    subplot(2, 2, 2);
    semilogy(w/w1, abs(ifft(E))); grid on;
    set(gca, 'xlim', [-5 5], 'xtick', [-5 : 5], 'ylim', [1e-20 1e10]);
    title('FFT(E)'); xlabel('Frequency (w_1)');
    
    subplot(2, 1, 2);
    semilogy(w/w1, abs(Cn), w(idx)/w1, abs(Cn(idx)), 'ro'); grid on;
    set(gca, 'xlim', [-8 8], 'xtick', [-8 : 8], 'ylim', [1e-20 1]);
    title('FFT(x)'); xlabel('Frequency (w_1)');
    
    
%     subplot(2, 2, 4);
%     plot(w1list(1:w1idx)/w0, abs(Cn1s)*5e5, 'ro'); hold on; grid on;
%     plot(w1list(1:w1idx)/w0, angle(Cn1s), 'Bs');
%     title('Amplitude and Phase'); xlabel('Frequency of E(t) (w_0)');
    
    drawnow;
